//
// Uart.cpp
// C++ code
// ----------------------------------
// Developed with embedXcode+
// http://embedXcode.weebly.com
//
// Project      MY_berr
//
// Created by   刘钊, 15/10/29 下午3:27
//
// Copyright    (c) 刘钊, 2015年
// Licence      <#license#>
//
// See          Uart.h and ReadMe.txt for references
//

#include "Uart.h"
#include "Arduino.h"

/****************************************************************************
 * 名称： WaitCommand()
 * 功能： 串口读数据
 * 参数： needwait, 传入char型1 或0
 ****************************************************************************/
unsigned char Uart::WaitCommand(char needwait) //传入数据,如果为0就循环检测串口
{
    int ttt = 0;                //延迟计数器.
    if (needwait)               //如果传入数据为1就延迟100毫秒.
    {
        while (Serial1.available() == 0)
        {
            delay(1);
            ttt++;
            if (ttt > 100)  //可改
            {
                return 0;   //超时退出
            }
        }
    }
    while (Serial1.available()) //监测串口是否有数据
    {
        _RecvBuff[_RecvCount++] = Serial1.read(); //串口读到得字节写入到SS
        
        ttt = 0;
        while (Serial1.available() == 0)
        {
            delayMicroseconds(100);
            ttt++;
            if (ttt > 20) //可改
                break;
        }
        //溢出处理
        if (_RecvCount >= _BufSize) //如果读入计数器大于等于64.
        {
            _RecvCount--; //计数器自减.
        }
    }
    /*if (_RecvCount > 0) //打印测试.
    {
        Serial.print("S:");
        for (int i = 0; i < _RecvCount; ++i)
        {
            Serial.print(_RecvBuff[i]);
            Serial.print(" ");
        }
        Serial.println(" ");
    }*/
    if ((_RecvBuff[_RecvCount - 3] == UART_End) &&
        (_RecvBuff[_RecvCount - 2] == UART_End) &&
        (_RecvBuff[_RecvCount - 1] == UART_End))
    {
        _SendCount = _RecvCount;
        _RecvCount = 0;
        return FindCommand();
    }
    _RecvCount = 0;
    return 0; //无有效命令
}
/****************************************************************************
 * 名称：FindCommand()
 * 功能：按照返回的指令处理数据;
 * 参数：rv;返回的常量参数
 ****************************************************************************/
unsigned char  Uart::FindCommand()
{
    unsigned char rv = 0;
    switch (_RecvBuff[0])
    {
            //有效指令
        case UART_touch:    UartTouch();        rv = UART_touch;        break; //触摸热区事件返回
        case UART_Touchdata:UartTouchdata();    rv = UART_Touchdata;    break; //触摸坐标数据返回
        case UART_Sleep:    UartSleep();        rv = UART_Sleep;        break; //眠模式触摸事件
        case UART_Ascii:    UartAscii();        rv = UART_Ascii;        break; //字符串变量数据返回
        case UART_Datavar:  UartDatavar();      rv = UART_Datavar;      break; //数值变量数据返回
        case UART_Sleepmode:UartSleepmode();    rv = UART_Sleepmode;    break; //设备自动进入睡眠模式
        case UART_SleepOpen:UartSleepOpen();    rv = UART_SleepOpen;    break; //设备自动唤醒
        case UART_Startup:  UartStartup();      rv = UART_Startup;      break; //系统启动成功
        case UART_SD:       UartSD();           rv = UART_SD;           break; //开始SD卡升级
            //无效指令
        case Error_CommandNO:ErrorCommand();    rv = Error_CommandNO;   break;  //无效指令
        case Error_CommandOK:ErrorCommand();    rv = Error_CommandOK;   break;  //指令成功执行
        case Error_ControlIdNO:ErrorCommand();  rv = Error_ControlIdNO; break;	//控件ID无效
        case Error_ScreenIdNO:ErrorCommand();   rv = Error_ControlIdNO; break;	//页面ID无效
        case Error_PicIdNO:   ErrorCommand();   rv = Error_PicIdNO;     break;  //图片ID无效
        case Error_FondNO:    ErrorCommand();   rv = Error_FondNO;      break;  //字库ID无效
        case Error_BeginNO:   ErrorCommand();   rv = Error_FondNO;      break;  //波特率设置无效
        case Error_CurveIdNO: ErrorCommand();   rv = Error_CurveIdNO;   break;  //曲线ID号通道无效
        case Error_VariableNO:ErrorCommand();   rv = Error_VariableNO;  break;  //变量名称无效
        case Error_OperationNO: ErrorCommand(); rv = Error_OperationNO; break;  //变量运算无效
        default:
            break;
    }
    //打印返回的函数
    Serial.print("FC:");
    Serial.print (rv);
    Serial.print("  =");
    for (int i = 0; i < _SendCount; ++i)
    {
        Serial.print(_RecvBuff[i]);
        Serial.print(" ");
    }
    Serial.println("");
    return rv;
}
/****************************************************************************
 * 名称：ButtonState()
 * 功能：检测按钮状态
 * 参数：EndButton,StartButtone ,PauseButton,ContinueButton
 ****************************************************************************/
void Uart::ButtonState()
{
    switch (cmd_screen_id)
    {
            //默认状态
        case 0:
            break;
            
            //页面一按钮事件
        case 1:
            switch (cmd_control_id)
        {
                //页面一按钮 2
            case 2:
                unsigned char Button1;
                Button1=GetButtonV(0);
                //如果为0,结束
                if (Button1==0)
                {
                    Serial.println("END");
                    EndButton=true;
                    StartButton=false;
                    cmd_control_id=0;
                    return;
                }
                //如果为1,开始
                if (Button1==1)
                {
                    Serial.println("START");
                    EndButton==false;
                    StartButton=true;
                    cmd_control_id=0;
                    return;
                }
                cmd_screen_id=0;
                break;
                //页面一按钮 3
            case 3:
                unsigned char Button2;
                Button2=GetButtonV(1);
                //如果为0,继续
                if (Button2==0)
                {
                    PauseButton=false;
                    ContinueButton=true;
                    Serial.println("Continue");
                    cmd_control_id=0;
                    return;
                }
                //如果为0,暂停
                if (Button2==1)
                {
                    PauseButton=true;
                    ContinueButton=false;
                    Serial.println("Pause");
                   cmd_control_id=0;
                    return;
                }
                cmd_screen_id=0;
                break;
            default:
                break;
        }
            break;//页面一结束;
            
        case 2:
            
            break;//页面二结束;
        case 3:
            break;
        default:
            break;
    }
}

/****************************************************************************
 * 名称：GetControlV
 * 功能：取数值数据;整个函数返回取值.get n().val
 * 参数：Contro;
 ****************************************************************************/
unsigned char Uart::GetControlV(unsigned char Contro)
{
    String Get = "get n", Val = ".val", GetValL;
    GetValL = Get + Contro + Val;
    Serial1.print(GetValL);
    UartEnd();
    unsigned char data;
    if (WaitCommand(1)==UART_Datavar)
    {
        data=GetContro_Return_val;
        return data;
    }
}
/****************************************************************************
 * 名称：GetControlV
 * 功能：取按钮状态数据;整个函数返回取值.get bt().val
 * 参数：Contro;
 ****************************************************************************/
unsigned char Uart::GetButtonV(unsigned char Contro)
{
    String Get = "get bt", Val = ".val", GetValL;
    GetValL = Get + Contro + Val;
    Serial1.print(GetValL);
    UartEnd();
    unsigned char data;
    if (WaitCommand(1)==UART_Datavar)
    {
        data= GetButton_Return_val;
        return data;
    }
}


/****************************************************************************
 * 名称：UartTouch()
 * 功能：触摸热区事件返回函数
 * 参数：
 ****************************************************************************/
unsigned char Uart::UartTouch()
{
    Serial.println("UARTOUCH");
    unsigned char returnv = 0;
    cmd_screen_id = _RecvBuff[1];
    returnv = cmd_screen_id;
    cmd_control_id = _RecvBuff[2];
    returnv = cmd_control_id;
    cmd_button_v = _RecvBuff[3];
    returnv = cmd_button_v;
    return returnv;
    
}

/****************************************************************************
 * 名称： UartTouchdata()
 * 功能： 触摸坐标数据返回函数
 * 参数：
 ****************************************************************************/
void Uart::UartTouchdata()
{
    
}
/****************************************************************************
 * 名称： 睡眠模式触摸事件函数
 * 功能： 函数集合
 * 参数：
 ****************************************************************************/
void Uart::UartSleep()
{
    
}
/****************************************************************************
 * 名称： UartAscii()
 * 功能： 字符串变量数据返回函数
 * 参数：
 ****************************************************************************/
void Uart::UartAscii()
{
    
}
/****************************************************************************
 * 名称： UartDatavar()
 * 功能： 数值变量数据返回函数
 * 参数：
 ****************************************************************************/
void Uart::UartDatavar()
{
    unsigned char data1 ;
    GetButton_Return_val=0;GetContro_Return_val=0;
    if (_SendCount >= 8)
    {
        GetButton_Return_val = _RecvBuff[_SendCount - 7];
        GetContro_Return_val = _RecvBuff[_SendCount - 7];
    }
}
/****************************************************************************
 * 名称： UartSleepmode()
 * 功能： 设备自动进入睡眠模式函数
 * 参数：
 ****************************************************************************/
void Uart::UartSleepmode()
{
    
}
/****************************************************************************
 * 名称： UartSleepOpen()
 * 功能： 设备自动唤醒函数
 * 参数：
 ****************************************************************************/
void Uart::UartSleepOpen()
{
    
}
/****************************************************************************
 * 名称： UartStartup()
 * 功能： 系统启动成功函数
 * 参数：
 ****************************************************************************/

void Uart::UartStartup()
{
    
}
/****************************************************************************
 * 名称： UartSD()
 * 功能： 开始SD卡升级函数
 * 参数：
 ****************************************************************************/
void Uart::UartSD()
{
    
}
/****************************************************************************
 * 名称： UartEnd();
 * 功能：  结束符函数
 * 参数： 无
 ****************************************************************************/
void Uart::UartEnd()
{
    Serial1.write(UART_End);
    Serial1.write(UART_End);
    Serial1.write(UART_End);
}
/****************************************************************************
 * 名称：ErrorCommand()
 * 功能：无效指令返回;
 * 参数：
 ****************************************************************************/
unsigned char Uart::ErrorCommand()
{
    if(_RecvBuff[0]==Error_CommandNO)    Serial.println("无效指令_0X00+结束符");
    if(_RecvBuff[0]==Error_CommandOK)    Serial.println("指令成功执行_	0X01+结束符");
    if(_RecvBuff[0]==Error_ControlIdNO)  Serial.println("控件ID无效_0X02+结束符");
    if(_RecvBuff[0]==Error_ScreenIdNO)   Serial.println("页面ID无效_0X03+结束符");
    if(_RecvBuff[0]==Error_PicIdNO)      Serial.println("图片ID无效_0X04+结束符");
    if(_RecvBuff[0]==Error_FondNO)       Serial.println("字库ID无效_0X05+结束符");
    if(_RecvBuff[0]==Error_BeginNO)      Serial.println("波特率设置无效_0X11+结束符");
    if(_RecvBuff[0]==Error_CurveIdNO)    Serial.println("曲线控件ID号或通道号无效_0X12+结束符");
    if(_RecvBuff[0]==Error_VariableNO)   Serial.println("变量名称无效_0X1A+结束符");
    if(_RecvBuff[0]==Error_OperationNO)  Serial.println("变量运算无效_0X1B+结束符");
}
/****************************************************************************
 * 名称 SendDim()
 * 功能：设置屏幕亮度0-100
 * 参数：i(0-100)
 ****************************************************************************/
void Uart::SendDim(int i)
{
    if (i<=100)
    {
        String Dim = "dim=", DimVal;
        DimVal = Dim+i;
        Serial1.print(DimVal);
        UartEnd();
    }
}