//
// Uart.cpp
// C++ code
// ----------------------------------
// Developed with embedXcode+
// http://embedXcode.weebly.com
//
// Project      MY_berr
//
// Created by   刘钊, 15/10/29 下午3:27
//
// Copyright    (c) 刘钊, 2015年
// Licence      <#license#>
//
// See          Uart.h and ReadMe.txt for references
//


#include "Uart.h"
#include "Arduino.h"

/****************************************************************************
 * 名称： WaitCommand()
 * 功能： 串口读数据
 * 参数： needwait, 传入char型1 或0
 ****************************************************************************/
unsigned char Uart::WaitCommand(char needwait) //传入数据,如果为0就循环检测串口
{
    int ttt = 0;                //延迟计数器.
    if (needwait)               //如果传入数据为1就延迟100毫秒.
    {
        while (Serial1.available() == 0)
        {
            delay(1);
            ttt++;
            if (ttt > 100)  //可改
            {
                return 0;   //超时退出
            }
        }
    }
    while (Serial1.available()) //监测串口是否有数据
    {
        _RecvBuff[_RecvCount++] = Serial1.read(); //串口读到得字节写入到SS
        
        ttt = 0;
        while (Serial1.available() == 0)
        {
            delayMicroseconds(100);
            ttt++;
            if (ttt > 20) //可改
                break;
        }
        //溢出处理
        if (_RecvCount >= _BufSize) //如果读入计数器大于等于64.
        {
            _RecvCount--; //计数器自减.
        }
    }
    
    if (_RecvCount > 0) //打印测试.
    {
        Serial.print("S:");
        for (int i = 0; i < _RecvCount; ++i)
        {
            Serial.print(_RecvBuff[i]);
            Serial.print(" ");
        }
        Serial.println(" ");
    }
    
    if ((_RecvBuff[_RecvCount - 3] == UART_End) &&
        (_RecvBuff[_RecvCount - 2] == UART_End) &&
        (_RecvBuff[_RecvCount - 1] == UART_End))
    {
        _SendCount = _RecvCount;
        _RecvCount = 0;
        return FindCommand();
    }
    
    _RecvCount = 0;
    return 0; //无有效命令
}
unsigned char  Uart::FindCommand()
{
    unsigned char rv = 0;
    switch (_RecvBuff[0])
    {
        case UART_touch:
            UartTouch();    rv = UART_touch;    break; //触摸热区事件返回
        case UART_Touchdata:
            UartTouchdata();rv = UART_Touchdata;break; //触摸坐标数据返回
        case UART_Sleep:
            UartSleep();    rv = UART_Sleep;    break; //眠模式触摸事件
        case UART_Ascii:
            UartAscii();    rv = UART_Ascii;    break; //字符串变量数据返回
        case UART_Datavar:
            UartDatavar();  rv = UART_Datavar;  break; //数值变量数据返回
        case UART_Sleepmode:
            UartSleepmode();rv = UART_Sleepmode;break; //设备自动进入睡眠模式
        case UART_SleepOpen:
            UartSleepOpen();rv = UART_SleepOpen;break; //设备自动唤醒
        case UART_Startup:
            UartStartup();  rv = UART_Startup;  break; //系统启动成功
        case UART_SD:
            UartSD();       rv = UART_SD;       break; //开始SD卡升级
            //无效指令
        case Invalid_CommandNO:
            InvalidCommand();rv = Invalid_CommandNO;    break;	//无效指令 	0X00+结束符
        case Invalid_CommandOK:
            InvalidCommand();rv = Invalid_CommandOK;    break;  //指令成功执行 	0X01+结束符
        case Invalid_ControlIdNO:
            InvalidCommand();rv = Invalid_ControlIdNO;  break;	//控件ID无效 	0X02+结束符
        case Invalid_ScreenIdNO:
            InvalidCommand();rv = Invalid_ControlIdNO;  break;	//页面ID无效 	0X03+结束符
        case Invalid_PicIdNO:
            InvalidCommand();rv = Invalid_PicIdNO;      break;  //图片ID无效 	0X04+结束符
        case Invalid_FondNO:
            InvalidCommand();rv = Invalid_FondNO;       break;  //字库ID无效 	0X05+结束符
        case Invalid_BeginNO:
            InvalidCommand();rv	= Invalid_FondNO;       break;  //波特率设置无效 	0X11+结束符
        case Invalid_CurveIdNO:
            InvalidCommand();rv = Invalid_CurveIdNO;    break;  //曲线ID号通道无效0X12+结束符
        case Invalid_VariableNO:
            InvalidCommand();rv =Invalid_VariableNO;    break;//变量名称无效 	0X1A+结束符
        case Invalid_OperationNO:
            InvalidCommand();rv =Invalid_OperationNO;   break;//变量运算无效 	0X1B+结束符
        default:
            break;
    }
    Serial.print("FC:");
    Serial.print (rv);
    Serial.print("  =");
    for (int i = 0; i < _SendCount; ++i)
    {
        Serial.print(_RecvBuff[i]);
        Serial.print(" ");
    }
    Serial.println("");
    return rv;
}
/****************************************************************************
 * 名称：InvalidCommand()
 * 功能：无效指令返回;
 * 参数：
 ****************************************************************************/
unsigned char Uart::InvalidCommand()
{
    if(WaitCommand(1)==Invalid_CommandNO);    Serial.println("Invalid_CommandNO");
    if(WaitCommand(1)==Invalid_CommandOK);    Serial.println("Invalid_CommandOK");
    if(WaitCommand(1)==Invalid_ControlIdNO);  Serial.println("Invalid_ControlIdNO");
    if(WaitCommand(1)==Invalid_ScreenIdNO);   Serial.println("Invalid_ScreenIdNO");
    if(WaitCommand(1)==Invalid_PicIdNO);      Serial.println("Invalid_PicIdNO");
    if(WaitCommand(1)==Invalid_FondNO);       Serial.println("Invalid_FondNO");
    if(WaitCommand(1)==Invalid_BeginNO);      Serial.println("Invalid_BeginNO");
    if(WaitCommand(1)==Invalid_CurveIdNO);    Serial.println("Invalid_CurveIdNO");
    if(WaitCommand(1)==Invalid_VariableNO);   Serial.println("Invalid_VariableNO");
    if(WaitCommand(1)==Invalid_OperationNO);  Serial.println("Invalid_OperationNO");
}
//取数值数据
unsigned char Uart::GetData(unsigned char data)
{
    String Get = "get n", Val = ".val", GetValL;
    GetValL = Get + data + Val;
    Serial1.print(GetValL);
    UartEnd();
}


//取按钮当前状态
unsigned char Uart::GetButtonV(unsigned char data)
{
    String Get = "get bt", Val = ".val", GetValL;
    GetValL = Get + data + Val;
    Serial1.print(GetValL);
    UartEnd();
    delay(200);
}

//触摸热区事件返回函数
unsigned char Uart::UartTouch()
{
    Serial.println("UARTOUCH");
    unsigned char returnv = 0;
    cmd_screen_id = _RecvBuff[1];
    returnv = cmd_screen_id;
    cmd_control_id = _RecvBuff[2];
    returnv = cmd_control_id;
    cmd_button_v = _RecvBuff[3];
    returnv = cmd_button_v;
    return returnv;
    
}

/****************************************************************************
 * 名称： UartTouchdata()
 * 功能： 触摸坐标数据返回函数
 * 参数：
 ****************************************************************************/
void Uart::UartTouchdata()
{
    
}
/****************************************************************************
 * 名称： 睡眠模式触摸事件函数
 * 功能： 函数集合
 * 参数：
 ****************************************************************************/
void Uart::UartSleep()
{
    
}
/****************************************************************************
 * 名称： UartAscii()
 * 功能： 字符串变量数据返回函数
 * 参数：
 ****************************************************************************/
void Uart::UartAscii()
{
    
}
/****************************************************************************
 * 名称： UartDatavar()
 * 功能： 数值变量数据返回函数
 * 参数：
 ****************************************************************************/
void Uart::UartDatavar()
{
    unsigned char data1 ;
    Serial.println("cs2");
    Return_val = 0;
    if (_SendCount >= 8)
    {
        Return_val = _RecvBuff[_SendCount - 7];
    }
}
/****************************************************************************
 * 名称： UartSleepmode()
 * 功能： 设备自动进入睡眠模式函数
 * 参数：
 ****************************************************************************/
void Uart::UartSleepmode()
{
    
}
/****************************************************************************
 * 名称： UartSleepOpen()
 * 功能： 设备自动唤醒函数
 * 参数：
 ****************************************************************************/
void Uart::UartSleepOpen()
{
    
}
/****************************************************************************
 * 名称： UartStartup()
 * 功能： 系统启动成功函数
 * 参数：
 ****************************************************************************/

void Uart::UartStartup()
{
    
}
/****************************************************************************
 * 名称： UartSD()
 * 功能： 开始SD卡升级函数
 * 参数：
 ****************************************************************************/
void Uart::UartSD()
{
    
}
/****************************************************************************
 * 名称： UartEnd();
 * 功能：  结束符函数
 * 参数： 无
 ****************************************************************************/
void Uart::UartEnd()
{
    Serial1.write(UART_End);
    Serial1.write(UART_End);
    Serial1.write(UART_End);
}
